<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * This view helper renders the messages that have been added by calling the
 * action helper {@see Zend_Controller_Action_Helper_FlashMessenger}.
 *
 * It supports various message types that can be specified by passing an array
 * to the Zend_Controller_Action_Helper_FlashMessenger where the message must
 * be the first and the type second element.
 *
 * Example:
 *
 * Zend_Controller_Action_Helper_FlashMessenger::addMessage(
 *     array(
 *         'This is an error',
 *         DZF_View_Helper_FlashMessenger::TYPE_ERROR
 *     )
 * );
 *
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_View_Helper_FlashMessenger extends \Zend_View_Helper_Abstract
{

    /**
     *
     * @var string
     */
    const TYPE_INFO = 'info';

    /**
     *
     * @var string
     */
    const TYPE_ERROR = 'error';

    /**
     *
     * @var string
     */
    const TYPE_SUCCESS = 'success';

    /**
     *
     * @var array
     */
    protected $_messageClasses = array(
        self::TYPE_INFO => self::TYPE_INFO,
        self::TYPE_ERROR => self::TYPE_ERROR,
        self::TYPE_SUCCESS => self::TYPE_SUCCESS
    );

    /**
     *
     * @var string
     */
    protected $_type;

    /**
     *
     * @var array
     */
    protected $_messages = null;

    /**
     *
     * @var string
     */
    protected $_defaultType = self::TYPE_INFO;

     /**
     *
     * @var string
     */
    protected $_class = 'flashMessenger';


    /**
     *
     * @param string $method
     * @param array $args
     *
     * @return DZF_View_Helper_FlashMessenger fluent interface
     */
    public function __call($method, $args)
    {
        $matches = array();

        if (preg_match('/^set(\w+?)MessageClass$/', $method, $matches)) {
            if (count($args) == 1 && is_string($args[0])) {
                $type = strtolower($matches[1]);
                if (isset($this->_messageClasses[$type])) {
                    $this->_messageClasses[$type] = $args[0];
                    return $this;
                }
            }
        }
        trigger_error(
            sprintf(
                'Call to undefined method %s::%s',
                get_class($this),
                $method
            ),
            E_USER_ERROR
        );

        return $this;
    }

    /**
     * Initializes the flash messenger helper with a given message type.
     *
     * @param string|null $type
     * @return DZF_View_Helper_FlashMessenger fluent interface
     * @throws \Zend_View_Exception
     */
    public function flashMessenger($type = null)
    {
        $this->setType($type);

        //retrieve messages from controller action helper
        if (null === $this->_messages) {
            /** @var Zend_Controller_Action_Helper_FlashMessenger $helper */
            $helper = \Zend_Controller_Action_HelperBroker::getStaticHelper('flashMessenger');
            $currentMessages = $helper->getCurrentMessages();
            $messages = $helper->getMessages();
            $helper->clearCurrentMessages();

            $this->_messages = $this->_parseMessages(
                array_merge(
                    $messages,
                    $currentMessages
                )
            );
        }
        return $this;
    }

    /**
     * Sets the message type that should be used for display.
     *
     * @param string|null $type
     * @return DZF_View_Helper_FlashMessenger fluent interface
     * @throws \Zend_View_Exception
     */
    public function setType($type)
    {
        if (null !== $type && !isset($this->_messageClasses[$type])) {
            throw new \Zend_View_Exception(
                sprintf(
                    "Message type '%s' does not exist",
                    $type
                )
            );
        }
        $this->_type = $type;
        return $this;
    }

    /**
     * Set class name for div to be displayed
     *
     * @param string $class
     * @return DZF_View_Helper_FlashMessenger fluent interface
     */
    public function setClass($class)
    {
        $this->_class = (string) $class;
        return $this;
    }

    /**
     * Set the default type that should used for messages that have not already
     * set a type.
     *
     * @param string $type
     * @return DZF_View_Helper_FlashMessenger fluent interface
     * @throws \Zend_View_Exception
     */
    public function setDefaultType($type)
    {
        if (!isset($this->_messageClasses[$type])) {
            throw new \Zend_View_Exception(
                sprintf(
                    "Message type '%s' does not exist",
                    $type
                )
            );
        }
        $this->_defaultType = $type;
        return $this;
    }

    /**
     * Generates the output of the flash message view helper.
     *
     * The output contains only messages from the currently set type.
     * If no type is set all existing messages will be rendered.
     *
     * @return string
     */
    public function toString()
    {
        $html = '';
        if (is_array($this->_messages) && count(($this->_messages))) {
            $class = '';
            if (!empty($this->_class)) {
                $class = ' class="'.$this->_class . '"';
            }
            $html = '<div' . $class . '>';
            if (null === $this->_type) {
                foreach ($this->_messages as $type => $message) {
                    $html .= $this->_getMessages($type, $message);
                }
            } else {
                $messages = array();
                if (isset($this->_messages[$this->_type])) {
                    $messages = $this->_messages[$this->_type];
                }
                $html .= $this->_getMessages(
                    $this->_type,
                    $messages
                );
            }
            $html.="</div>";
        }

        // cleanup internal message queue after usage to avoid sending the
        // same message twice
        $this->_messages = null;

        return $html;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return $this->toString();
    }

    /**
     * Creates the output of messages with a specific type.
     *
     * @param string $type
     * @param array $messages
     * @return string
     */
    protected function _getMessages($type, $messages)
    {
        $html = '';

        if (!empty($messages)) {
            $messageClass = '';
            if (!empty($this->_messageClasses[$type])) {
                $messageClass = ' class="' . $this->_messageClasses[$type] .'"';
            }
            $html .= '<ul' . $messageClass .'>';
            foreach ($messages as $message) {
                $html .= '<li>' . $this->view->escape($this->view->translate($message)) . "</li>";
            }
            $html .= '</ul>';
        }
        return $html;
    }


    /**
     * Parses a given array of messages.
     *
     * Extracts the message type if existent or sets the default message type
     * for each message.
     *
     * @param array $messages
     *
     * @return array
     */
    protected function _parseMessages($messages)
    {
        $result = array();
        foreach ($messages as $message) {
            //create an array containing only the message text
            if (is_string($message)) {
                $message = (array) $message;
            }
            if (is_array($message)) {
                if (isset($message[0])) {
                    $text = (string) $message[0];
                    if (!empty($text)) {
                        //inially set default type
                        $type = $this->_defaultType;
                        //check if a specific type exists
                        if (
                            isset($message[1]) &&
                            isset($this->_messageClasses[(string) $message[1]])
                        ) {
                            $type = $message[1];
                        }
                        //create an entry in the result array if not existent
                        if (!isset($result[$type])) {
                            $result[$type] = array();
                        }
                        //add message to result
                        $result[$type][] = $text;
                    }
                }
            }
        }
        return $result;
    }
}